<?php

namespace Ems\Logic\ProcessamentoSolicitacao;

use Ems\Util\Utils;
use Util\{ DateUtil, NovoCodigo, Repositories };

use Teknisa\Model\Gestaopessoas\Entities\GpeTipotransfer;
use Teknisa\Model\Geral\Entities\Tipoestrutura;

class ProcessamentoSolicitacaoDesligamento extends ProcessamentoSolicitacao {
    
    const ALTERA_VINCULO = 'ALTERA';
    
    public function efetivar() {
        $itens = $this->getItensAssociativeArray();
        
        $nrvinculom         = $this->getSolicitacao()->getNrvinculom();
        $nrsolicitacao      = $this->getSolicitacao()->getNrsolicitacao();
        $nrorg              = Utils::getParamters()->getNrorg();
        $dtcompetencia      = DateUtil::getDataDeString(Utils::getParamters()->getDtmescomptencDiaCorrente(),DateUtil::FORMATO_BRASILEIRO,true);
   	    $gpevinculom        = Utils::getEntityManager()->getRepository(Repositories::GPE_VINCULOM)->findOneBy(array("nrvinculom" => $nrvinculom,"nrorg" => $nrorg));
        $gpevinculoH        = ProcessamentoSolicitacaoUtil::retornaUltimoHistoricoVinculoCompetencia($nrorg, $nrvinculom, $dtcompetencia->format('d/m/Y'));
        $nrestruturalegal   = $gpevinculoH->getNrestrutlegal();
        
        $nrtpavipre       = $itens['NRTPAVIPRE']      ?? null;
        $nrtpdemissao     = $itens['NRTPDEMISSAO']    ?? null;
        $nrmotivoresc     = $itens['NRMOTIVORESC']    ?? null;
        $dtavisoprevio    = $itens['DTAVISOPREVIO']   ?? null;
        $dtrescisaovinc   = $itens['DTRESCISAOVINC']  ?? null;
        $cdpessoanaograta = $itens['CDPESSOANAOGRATA'] ?? 'N';
        $dsRepVaga        = Utils::getSOrNText($itens['CDREPOSICAOVAGA']  ?? '');
        $dsPesNGrata      = Utils::getSOrNText($itens['CDPESSOANAOGRATA']  ?? '');
        $dsPossuiBens     = Utils::getSOrNText($itens['CDPOSSUIBENSDAEMPRESA']  ?? '');
        $dsobservacao     = isset($itens['DSOBSERVACAODESLIGAMENTO']) ? ' Justificativa: ' . $itens['DSOBSERVACAODESLIGAMENTO'] : '';

        $textoObsRescisao = <<<EOD
Rescisão realizada pela aprovação da solicitação de Nº $nrsolicitacao.
Reposição de Vaga: $dsRepVaga
Pessoa não Grata: $dsPesNGrata
Possui bens da Empresa: $dsPossuiBens
$dsobservacao
EOD;

    $textoObservacaoRescisao = <<<EOD
Rescisão realizada pela aprovação da solicitação de Nº $nrsolicitacao.
$dsobservacao
EOD;
        
        if($gpevinculom->getDtrescisaovinc() && $dtrescisaovinc){
            $acao = "ALTERA";
            $this->manipulaMovimentacao($nrorg, $nrvinculom, $nrestruturalegal, null, $dtrescisaovinc,$dtcompetencia->format('d/m/Y'), $acao, $nrtpdemissao);
        }else if(!$gpevinculom->getDtrescisaovinc() && $dtrescisaovinc){
            $acao = "INSERE";
            $this->manipulaMovimentacao($nrorg, $nrvinculom, $nrestruturalegal, null, $dtrescisaovinc,$dtcompetencia->format('d/m/Y'), $acao, $nrtpdemissao);
        }
        
        $this->manipulaSituacaoFuncional($dtrescisaovinc,$nrorg,$nrvinculom,$gpevinculom->getDtadmissaovinc(),$acao,null);

        $dtAvisoPrevio = !empty($dtavisoprevio)
            ? DateUtil::getDataDeString(substr($dtavisoprevio,0,10), DateUtil::FORMATO_BRASILEIRO, true)
            : null;
        
        $gpevinculom->setNrtpavipre($nrtpavipre);
        $gpevinculom->setNrtpdemissao($nrtpdemissao);
        $gpevinculom->setNrmotivoresc($nrmotivoresc);
        $gpevinculom->setDtavisoprevio($dtAvisoPrevio);
        $gpevinculom->setDtrescisaovinc(DateUtil::getDataDeString(substr($dtrescisaovinc,0,10), DateUtil::FORMATO_BRASILEIRO, true));
        $gpevinculom->setDsobsrescisao($textoObsRescisao);
        $gpevinculom->setDtultatu(DateUtil::getDataAtual(false));
        $gpevinculom->setNrorgultatu(Utils::getParamters()->getNrorg());
        $gpevinculom->setCdoperultatu(Utils::getParamters()->getCdoperador());
        
        Utils::getEntityManager()->persist($gpevinculom);
        Utils::getEntityManager()->flush();

        ProcessamentoSolicitacaoUtil::atualizaVinculo($nrorg, $nrvinculom, $cdpessoanaograta, $textoObservacaoRescisao);
    }
    
    public function rejeitar() {}

    public function atualizaVinculo($nrorg, $nrvinculom, $idverificarescisao, $dsobservacaorescisao) {
        $sql = <<<SQL
        UPDATE GPE_VINCULOM VM
        SET VM.IDVERIFICARESCISAO = :P_IDVERIFICARESCISAO,
            VM.DSOBSERVACAORESCISAO = :P_DSOBSERVACAORESCISAO
        WHERE VM.NRORG = :P_NRORG AND VM.NRVINCULOM = :P_NRVINCULOM
SQL;

        return Utils::getEntityManager()->getConnection()->prepare($sql)
            ->execute(array(
                "P_NRORG"                => $nrorg,
                "P_NRVINCULOM"           => $nrvinculom,
                "P_IDVERIFICARESCISAO"   => $idverificarescisao,
                "P_DSOBSERVACAORESCISAO" => $dsobservacaorescisao
            )
        );
    }
    
    public function manipulaMovimentacao(
        $nrorg, $nrVinculom, $nrEstruturaLegal, $nrEstruturaGerencial, $dtInicioMovimentacao, $mesCompetencia, $acao, $nrtpdemissao = null
    ) {
        try{
            Utils::getEntityManager()->beginTransaction();
            $nrTpmovtransfm = $this->setTipoRecisao($nrtpdemissao);
            $dtInicioMovimentacao = DateUtil::getDataDeString($dtInicioMovimentacao, DateUtil::FORMATO_BRASILEIRO, true);
            $mesCompetencia = DateUtil::getDataDeString($mesCompetencia, DateUtil::FORMATO_BRASILEIRO, true);
            
            if ($nrEstruturaLegal == null) {
                $gpevinculoh = ProcessamentoSolicitacaoUtil::retornaUltimoHistoricoVinculoCompetencia($nrorg, $nrVinculom);
                $nrEstruturaLegal = $gpevinculoh->getNrestrutlegal();
            }

            $gpemovimentacao = Utils::getEntityManager()->getRepository(Repositories::GPE_MOVIMENTACAO)->findOneBy(array('nrorg' => $nrorg,
                   'nrvinculom' => $nrVinculom,'nrtpmovtransfm' => array(3,4,5,6,7,11)),(array('dtinimoviment' => 'DESC')));
            if (is_object($gpemovimentacao)) {
                if (!$dtInicioMovimentacao) {
                    Utils::getEntityManager()->remove($gpemovimentacao);
                    Utils::getEntityManager()->flush();
                    Utils::getEntityManager()->commit();
                    $this->setReservaValeTransporte($nrorg, $nrVinculom, 'N');
                    return;
                } else if ($dtInicioMovimentacao != $gpemovimentacao->getDtinimoviment() || $nrTpmovtransfm != $gpemovimentacao->getNrtpmovtransfm() ) {
                    $gpemovimentacao->setDtinimoviment($dtInicioMovimentacao);
                    $gpemovimentacao->setNrtpmovtransfm($nrTpmovtransfm);
                    $this->setReservaValeTransporte($nrorg, $nrVinculom, 'S');
                    Utils::getEntityManager()->persist($gpemovimentacao);
                    Utils::getEntityManager()->flush();
                    Utils::getEntityManager()->commit();
                    return;
                }
            } 
            if($acao !== self::ALTERA_VINCULO && $dtInicioMovimentacao) {
                $paramOrg = Utils::getEntityManager()->getRepository(Repositories::GPE_PARAMORG)->findOneBy(array('nrorg' => Utils::getParamters()->getNrorg()));
                if ($nrEstruturaLegal) {
                    $movimentacaoLegal = $this->incluiMovimentacao($nrorg,GpeTipotransfer::TIPOTRANSFLEGAL, $nrVinculom, $nrEstruturaLegal, $dtInicioMovimentacao, $paramOrg->getNrtpestrutlegal(), $nrTpmovtransfm);
                }
                if ($nrEstruturaGerencial) {
                    $movimentacaoGeren = $this->incluiMovimentacao($nrorg,GpeTipotransfer::TIPOTRANSFGEREN, $nrVinculom, $nrEstruturaGerencial, $dtInicioMovimentacao, $paramOrg->getNrtpestrutgeren(), $nrTpmovtransfm);
                }
                $this->setReservaValeTransporte($nrorg, $nrVinculom, 'S');
            }
            else if($dtInicioMovimentacao && $acao === self::ALTERA_VINCULO) {
                if (DateUtil::comparaDatas($dtInicioMovimentacao,$mesCompetencia)) {
                    // alteracao
                    if ($nrEstruturaLegal) {
                        $this->alteraMovimentacao($nrorg, $mesCompetencia, $nrVinculom, $nrEstruturaLegal, $dtInicioMovimentacao, GpeTipotransfer::TIPOTRANSFLEGAL);
                    }

                    if ($nrEstruturaGerencial) {
                        $this->alteraMovimentacao($nrorg, $mesCompetencia, $nrVinculom, $nrEstruturaGerencial, $dtInicioMovimentacao, GpeTipotransfer::TIPOTRANSFGEREN);
                    }
                    $this->setReservaValeTransporte($nrorg, $nrVinculom, 'S');
                }
            }
            unset($dtAux);
            unset($vinculoLogic);
            Utils::getEntityManager()->flush();
            Utils::getEntityManager()->commit();

        }catch (\Exception $e){
            Utils::getEntityManager()->rollback();
            throw $e;
        }
    }
    
    public function setTipoRecisao($nrtpdemissao){
        switch($nrtpdemissao)
        {
            case 1;
                return 4;
            case 2;
                return 3;
            case 3;
                return 11;
            case 4;
                return 4;
            case 5;
                return 5;
            case 6;
                return 12;
            case 7;
                return 12;
            case 10;
                return 7;
            case 11;
                return 7;
            case 12;
                return 7;
            case 13;
                return 6;
            case 14;
                return 6;
            case 15;
                return 6;
            case 16;
                return 6;
            case 17;
                return 6;
            case 18;
                return 6;
            case 19;
                return 6;
            case 21;
                return 5;
            case 24;
                return 5;
            case 25;
                return 5;

            default;
                return 3;
        }
    }
    
    public function setReservaValeTransporte($nrorg, $nrVinculom, $idreserva){
        
        $vablinhavincs = Utils::getEntityManager()->getRepository(Repositories::VAB_LINHAVINC)->findOneBy(
            [
                'nrorg'      => $nrorg,
                'nrvinculom' => $nrVinculom
            ]
        );
        if($vablinhavincs){
            foreach ($vablinhavincs as $vablinhavinc){
                
                $vabintegtransphs = Utils::getEntityManager()->getRepository(Repositories::VAB_INTEGTRANSPH)->findOneBy(
                    [
                        'nrorg'           => $nrorg,                       
                        'nrtipolinhavinc' => $vablinhavinc->getNrtipolinhavinc(),
                        'nrintegtranspm'  => $vablinhavinc->getNrintegtranspm()
                    ]
                );
                
                if($vabintegtransphs){    
                    foreach ($vabintegtransphs as $vabintegtransph){
                        $vabintegtransph->setIdintegreserva($idreserva);
                        Utils::getEntityManager()->persist($vabintegtransph);
                        Utils::getEntityManager()->flush();
                    }
                }    
            }
        }    
    }
    
    /**
     * Inclui movimentacao do vinculo.
     *
     * @param integer    $nrOrg            Numero da Organizacao.
     * @param integer    $nrTipoTransfer   Numero do Tipo de transferencia.
     * @param integer    $nrVinculom       Numero do Vinculo.
     * @param integer    $nrEstruturam     Numero da Estrutra.
     * @param string     $dtIniMoviment    Data inicial da movimentacao do vinculo.
     * @param integer    $nrTipoEstrutura  Numero do Tipo de Estrutura.
     * @param integer    $nrTpMovTransf    Numero do Tipo de Movimentacao de Transferencia.
     * @param \DateTime  $dtRetroativa     Data Retroativa.
     * @param string     $idAtualizaAcerto Identificar de atualizacao de acerto (normal = null, excluir = 1 ou incluir = 2).
     * @param string     $dtFimMovimento   Data final da movimentacao do vinculo.
     *
     * @return $this->gpeMovimentacao.
     */
    public function incluiMovimentacao(
        $nrOrg, $nrTipoTransfer, $nrVinculom, $nrEstruturam, $dtIniMoviment, $nrTipoEstrutura = null,
        $nrTpMovTransf = null, $nrMotivoTransf = null, $dtRetroativa = null, $idAtualizaAcerto = 0,
        $dtFimMovimento = null, $dsObservacao = null
    ){
        
        $gpeMovimentacao = Repositories::GPE_MOVIMENTACAO;
        $movimentacao = new $gpeMovimentacao;
        $movimentacao->setNrorg($nrOrg);
        $movimentacao->setNrtipotransfer($nrTipoTransfer);
        $movimentacao->setNrvinculom($nrVinculom);
        $movimentacao->setNrestruturam($nrEstruturam);
        $movimentacao->setDtinimoviment($dtIniMoviment);
        $movimentacao->setNrtipoestrutura($nrTipoEstrutura);
        $movimentacao->setNrtpmovtransfm($nrTpMovTransf);
        $movimentacao->setNrmotivotransf($nrMotivoTransf);
        $movimentacao->setDtmovimentretro($dtRetroativa);
        $movimentacao->setIdatualizaacerto($idAtualizaAcerto);
        $movimentacao->setDtfimmoviment($dtFimMovimento);
        $movimentacao->setDsobservacao($dsObservacao);

        $movimentacao->setNrorginclusao(Utils::getParamters()->getNrorg());
        $movimentacao->setDtinclusao(DateUtil::getDataAtual());
        $movimentacao->setCdoperinclusao(Utils::getParamters()->getCdoperador());
        $movimentacao->setNrorgultatu(Utils::getParamters()->getNrorg());
        $movimentacao->setDtultatu(DateUtil::getDataAtual());
        $movimentacao->setCdoperultatu(Utils::getParamters()->getCdoperador());

        $movimentacao->setNrmovimentacao(NovoCodigo::geraCodigo(Utils::getParamters()->getNrorg(), 'GPE_MOVIMENTACAO', 1, 12, true));

        Utils::getEntityManager()->persist($movimentacao);
        Utils::getEntityManager()->flush();

        // Retorna movimentacao
        return $movimentacao;
    }
    
    public function manipulaSituacaoFuncional(
        $dtrescisaoatual, $nrorg, $nrvinculom, $dtIniSituacaoFunc, $acao, $nrmotivoafasta, $nrsitufunc = 1, $dtrescisaohist = null)
    {
        switch ($acao) {
            case 'INSERT':
                if (!$nrmotivoafasta) {
                    throw new \Exception('Selecione o motivo de afastamento.',10);
                }
                $this->incluiAlteSituFunc($nrorg,$nrvinculom, $nrsitufunc, $dtIniSituacaoFunc);
                break;
            case 'ALTERA':
                return self::regraMudarDataRescisao($nrorg, $nrvinculom, $dtrescisaoatual, $nrmotivoafasta, $dtrescisaohist);
                break;
        }
    }
    
    public function incluiAlteSituFunc($nrorg, $nrvinculom, $nrSituacaoFuncionario, $dtIniSituacaoFunc,
                         $motivoAfastamento = null, $cdDiagnostico = null, $codigoCDI = null,
                         $dtFimSituacaoFunc = null, $dtPreviretoSitu = null, $dtProrrSituacaoFunc = null,
                         $dtVerifSituacaoFunc = null, $nrProxSituFunc = null, $qntDias = null, $dsObs = null)
    {
        /** @var \Teknisa\Model\Gestaopessoas\Entities\GpeAltesitufunc $altesitufunc */
        
        $gpeAltesitufunc = Repositories::GPE_ALTESITUFUNC;
        $altesitufunc = new $gpeAltesitufunc;
        
        $altesitufunc->setNrorg($nrorg);
        $altesitufunc->setNrvinculom($nrvinculom);
        $altesitufunc->setNrsitufuncm($nrSituacaoFuncionario);
        $altesitufunc->setDtinisitufunc($dtIniSituacaoFunc);
        $altesitufunc->setNrqtdiasitufunc($qntDias);

        $altesitufunc->setNrmotivoafasta($motivoAfastamento);
        $altesitufunc->setCddiagnost($cdDiagnostico);
        $altesitufunc->setCdtabecdi($codigoCDI);
        $altesitufunc->setDtfimsitufunc($dtFimSituacaoFunc);
        $altesitufunc->setDtpreviretositu($dtPreviretoSitu);
        $altesitufunc->setDtprorrsitufunc($dtProrrSituacaoFunc);
        $altesitufunc->setDtverifsitufunc($dtVerifSituacaoFunc);
        $altesitufunc->setNrproxsitufunc($nrProxSituFunc);
        $altesitufunc->setDsObsAltSitFunc($dsObs);

        $altesitufunc->setNrorginclusao($nrorg);
        $altesitufunc->setDtinclusao(DateUtil::getDataAtual());
        $altesitufunc->setCdoperinclusao(Utils::getParamters()->getCdoperador());
        $altesitufunc->setNrorgultatu($nrorg);
        $altesitufunc->setDtultatu(DateUtil::getDataAtual());
        $altesitufunc->setCdoperultatu(Utils::getParamters()->getCdoperador());

        $altesitufunc->setNraltesitufunc(NovoCodigo::geraCodigo(Utils::getParamters()->getNrorg(), 'GPE_ALTESITUFUNC', 1, 12, true));

        Utils::getEntityManager()->persist($altesitufunc);
        return $altesitufunc;
    }
    
    public function regraMudarDataRescisao($nrorg, $nrvinculom, $dtrescisao, $nrmotivoafasta, $dtrescisaohist = null)
    {
        try {
            Utils::getEntityManager()->beginTransaction();
            $nrsitufunc = 13;
            $nr = null;
            if($dtrescisaohist){
                $dtultimodia = $dtrescisaohist;
            }else {
                $dtultimodia = Utils::getParamters()->getUltimoDiaMesCorrente();
            }
            $ultimaSituacao = $this->retornaUltimaSituacaoPorDatainicio($nrorg, $nrvinculom, $dtultimodia);
            if (!$ultimaSituacao) {
                throw new \Exception('Vinculo sem situacao funcional!');
            }
            
            if (!$dtrescisao && $ultimaSituacao->getNrsitufuncm() == 13) { // Situacao Demitido
                // Retirar rescisao
                $dtrescis = $ultimaSituacao->getDtinisitufunc();
                Utils::getEntityManager()->beginTransaction();
                Utils::getEntityManager()->remove($ultimaSituacao);
                Utils::getEntityManager()->flush();
                Utils::getEntityManager()->commit();
                $penultima = $this->retornaUltimaSituacaoPorDatainicio($nrorg, $nrvinculom, $dtultimodia);
                if($penultima && $penultima->getNrsitufuncm() == 1 && $penultima->getDtfimsitufunc() && $penultima->getDtfimsitufunc()->modify('+ 1 day')->format('d/m/Y') == $dtrescis->format('d/m/Y')){
                    $penultima->setDtfimsitufunc(null);
                }
            } else if ($dtrescisao) {
                // Alterar data de rescisao
                $dtrescisao = DateUtil::getDataDeString($dtrescisao,DateUtil::FORMATO_BRASILEIRO, true);
                
                if ($ultimaSituacao->getNrsitufuncm() == 13) {
                    $ultimaSituacao->setDtinisitufunc($dtrescisao);
                    $nr = $ultimaSituacao->getNrsitufuncm();
                } else {
                    // Cria nova situacao de rescisao
                    $altesitufunc = $this->incluiAlteSituFunc($nrorg,$nrvinculom, $nrsitufunc, $dtrescisao, $nrmotivoafasta);
                    $dtrescisao = clone $dtrescisao;
                    $dtrescisao->modify('-1 day');
                    if(!$ultimaSituacao->getDtfimsitufunc()) {
                        $ultimaSituacao->setDtfimsitufunc($dtrescisao);
                    }
                    $nr = $altesitufunc->getNrsitufuncm();
                }
            }
            Utils::getEntityManager()->flush();
            Utils::getEntityManager()->commit();
            if($nr){
                return $nr;
            }
        } catch (\Exception $e) {
            Utils::getEntityManager()->rollback();
            throw $e;
        }
    }
    
    public function retornaUltimaSituacaoPorDatainicio($nrorg, $nrvinculom, $dtinisitufunc) {
        $gpeAltesitufunc = Repositories::GPE_ALTESITUFUNC;
        $sql = <<<SQL
SELECT a
  FROM $gpeAltesitufunc a
WHERE a.nrorg         = :NRORG
  AND a.nrvinculom    = :NRVINCULOM
  AND a.dtinisitufunc = (SELECT MAX(b.dtinisitufunc)
                           FROM $gpeAltesitufunc b
                          WHERE b.nrorg       = a.nrorg
                            AND b.nrvinculom  = a.nrvinculom
                            AND b.dtinisitufunc <= :DTINISITUFUNC)
SQL;
        $params = array(
            'NRORG' => $nrorg,
            'NRVINCULOM' => $nrvinculom,
            'DTINISITUFUNC' => $dtinisitufunc
        );
        $result = Utils::getEntityManager()->createQuery($sql)->execute($params);
        return empty($result) ? $result : $result[0];
    }
    
    public function alteraMovimentacao($nrorg, $mesCompetencia, $nrVinculom, $nrEstrutura, $dtInicioMovimentacao, $tpTransferencia){
        try{
            
            $objEstrutura = Utils::getEntityManager()->getRepository(Repositories::ESTRUTURAM)->findOneBy(array('nrorg' => $nrorg,
                   'nrestruturam' => $nrEstrutura));
            $nrTipoEstrutura = $objEstrutura->getNrtipoestrutura();
            unset($objEstrutura);
            
            $listaMovimentacoes = Utils::getEntityManager()->getRepository(Repositories::GPE_MOVIMENTACAO)->findBy(array('nrorg' => $nrorg,
                   'nrvinculom' => $nrVinculom));
            foreach ($listaMovimentacoes as $altMovimentacao){
                if ($dtInicioMovimentacao === $altMovimentacao->getDtinimoviment()
                    && $altMovimentacao->getNrestruturam() !== $nrEstrutura
                    && $altMovimentacao->getNrtipotransfer() === (string)$tpTransferencia
                ){
                    $this->verificaTipoTransferencia($tpTransferencia, $nrTipoEstrutura);
                    $altMovimentacao->setNrestruturam($nrEstrutura);
                    $altMovimentacao->setNrtipoestrutura($nrTipoEstrutura);
                    $altMovimentacao->persist();
                }
            }
            unset($movimentacao);
            unset($paramorg);

        }catch (Exception $e){
            throw $e;
        }
    }
    
    public function verificaTipoTransferencia($nrtipotransfer, $nrtipoestrutura){
        try {
            $paramOrg = Utils::getEntityManager()->getRepository(Repositories::GPE_PARAMORG)->findOneBy(array('nrorg' => Utils::getParamters()->getNrorg()));
            if ($nrtipotransfer == GpeTipotransfer::TIPOTRANSFLEGAL) {
                if ($nrtipoestrutura != $paramOrg->getNrtpestrutlegal() && $nrtipoestrutura != Tipoestrutura::SINDICATO) {
                    throw new \Exception('Movimentaçao legal incompatível com o tipo de estrutura', 30);
                }
            } elseif ($nrtipotransfer == GpeTipotransfer::TIPOTRANSFGEREN) {
                if (($nrtipoestrutura == $paramOrg->getNrtpestrutlegal() && $paramOrg->getNrtpestrutlegal() != $paramOrg->getNrtpestrutgeren()) || $nrtipoestrutura == Tipoestrutura::SINDICATO) {
                    throw new \Exception('Movimentaçao gerencial incompatível com o tipo de estrutura', 30);
                }
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }
}