<?php

namespace Ems\Services;

use Doctrine\ORM\EntityManager;

use Ems\Services\{ EmailService, SolicitacaoService };
use Util\{ EmailMessages, Environment, NovoCodigo, DateUtil };
use Ems\Repository\VinculoRepository;

use Ems\Repository\MinhaHistoriaRepository;
use Ems\Repository\SolicitacaoRepository;
use Ems\Util\{ Utilities, ArrayUtil, Utils };

class MinhaHistoriaService {

    private $default_key = ['NRORG','NRVINCULOM','NRTIPOSOLICITACAO','DSOBSSOLICITACAO', 'DTSOLICITACAO'];

    private $labelFieldPessoa = [
        "NRORG" => "Organização", "DTMESCOMPETENC" => "Competência", "DTESOCIAL" => "Data Esocia", "NRPESSOA" => "Nr Pessoa", "NRPARCNEGOCIO" => "Nr Parceiro Negocio", "NRPESSOAH" => "Nr Historico", "DSPRONOMETRAT" => "Tratamento", "NMPESSOA" => "Nome",
        "DSAPELIDOPESS" => "Apelido", "DTNASCPESSOA" => "Dt. Nasc.", "IDSEXOPESSOA" => "Sexo", "NMESTACIVIL" => "Estado Civil", "CDESTACIVIL" => "CD. Estado Civil", "IDGRUPSANGUPES" => "Grupo Sanguíneo", "IDFATORRHPES" => "Fator RH", "DSCONDFISPES" => "Condição Física",
        "NRCONDFISPES" => "Nr. Condição Física", "IDCOTAPCD" => "Cota PCD", "NMRACAPESSOA" => "Raça", "NRRACAPESSOA" => "Nr. Raça", "NMGRAUINSTR" => "Grau de Instrução", "NRGRAUINSTR" => "Nr. Grau de Instrução", "NMPAIS" => "País", "CDMUNICIPIO" => "Cd. Município",
        "NMMUNICIPIO" => "Município", "NRNACIONALIDADE" => "Nr. Nacionalidade", "DSNACIONALIDADE" => "Nacionalidade", "DTADMISSAOPRE" => "Dt. Admissão Preliminar", "CDRECIBOADMPRE" => "Nº Recibo Admissão Preliminar", "DSEXPDOCESTR" => "Órgão emissor da RNE",
        "DTCHEGADAPAIS" => "Dt. Chegada País", "NRDOCUESTRANG" => "Doc. Estrangeiro", "IDPESNATURALIZA" => "Naturalizado", "DTNATURALIZAPES" => "Dt. Naturalidade", "IDPOSVISTOPERM" => "Visto Permanente", "DTPERMAVISTO" => "Dt. Permanência", "NRCPFPESSOA" => "CPF",
        "NRRGPESSOA" => "RG", "CDEXRGPESSOA" => "Cd. Órgão Expedidor RG", "NMEXRGPESSOA" => "Órgão Expedidor RG", "DTEXRGPESSOA" => "Dt. Expedição RG", "DSLOCALEXRGPES" => "Local de Expedição", "SGUFRGPESSOA" => "UF-RG", "NRSUSPESSOA" => "Nr. Cartão SUS", "NRNITPESSOA" => "NIT",
        "NRZONAELEIPES" => "Zona Eleitoral", "NRSECAELEIPES" => "Seção Eleitoral", "IDISENTITUELEI" => "Isento Título Eleitoral", "NRCERTNASCPES" => "Certidão Nascimento", "NRLIVROCERTNASC" => "Livro Certidão Nascimento", "NRFOLHACERTNASC" => "Folha Certidão Nascimento",
        "NRCERTCASAPES" => "Certidão Casamento", "DTCASAPESSOA" => "Dt. Casamento", "NRCARTHABPES" => "CNH", "DTHABCARTPES" => "Dt. Habilitação", "DSCATEGHABCART" => "Categoria CNH", "DTVALHABCARTPES" => "Validade CNH", "SGUFHABCARPES" => "UF CNH", "NRCTPSPESSOA" => "Nr. CTPS",
        "NRSERIECTPSPES" => "Série CTPS", "SGUFCTPSPES" => "UF do CTPS", "DTCTPSPESSOA" => "Dt. Emissão CTPS", "NRPISPASEPPES" => "Nr. PIS/PASEP", "DTINSCPISPASEP" => "Dt. PIS/PASEP", "NRINSCINSSPES" => "INSS", "NRINSCISSPES" => "ISS", "NRREGCTPSPES" => "Registro Profissional MTE",
        "NMCONSELHOREG" => "Conselho Regional", "NRCONSELHOREG" => "Nr. Conselho Regional", "NRINSCRCONSREG" => "Inscrição no Conselho", "DTEXCONSREG" => "Dt. Ex. Conselho", "NMUFCONSREG" => "UF Conselho", "CDPAIS" => "Cd. País", "SGESTADO" => "Sg. Estado", "NMESTADO" => "Estado",
        "DTINIVIGENCIA" => "Início Vigência Conselho", "DTFIMVIGENCIA" => "Fim Vigência Conselho", "NRCATMILITARPE" => "Nr. Categoria Militar", "NMCATMILITARPE" => "Categoria Militar", "NRCERTRESEPES" => "Certificado de Reservista",
        "CDBANCO" => "Nr. Banco", "NMBANCO" => "Nm. Banco", "CDAGENCIA" => "Cd. Agencia Bancaria", "NMAGENCIA" => "Nm. Agencia Bancaria", "NRTPCONTCORR" => "Nr. Tipo Conta Corrente", "DSTPCONTCORR" => "Tipo Conta Corrente", "CDCONTCORR" => "Conta Corrente", "NRCEPENDERECO" => "Número do CEP",
        "CDTIPOENDERECO" => "CD. Tipo Endereço", "NMTIPOENDERECO" => "Nm. Tipo Endereço", "NMBAIRROENDERECO" => "Bairro", "CDLOGRADOURO" => "Cd. Logradouro", "DSLOGRADOURO" => "Ds. Logradouro", "DSENDERECO" => "Endereço", "NRIMOVELENDERECO" => "Nr Imovel", "DSCOMPLEENDERECO" => "Complemento Imovel",
        "DSREFERENCIAENDE" => "Referencia Imovel", "NRTPAVIPRE" => "Tipo de Aviso Prévio", "NRTPDEMISSAO" => "Tipo de Demissão", "NRMOTIVORESC" => "Motivo da Rescisão", "DTAVISOPREVIO" => "Dt. de aviso prévio", "DTRESCISAOVINC" => "Dt. de Rescisão Vínculo", "CDREPOSICAOVAGA" => "Reposição de Vaga", "CDPESSOANAOGRATA" => "Pessoa não Grata", "CDPOSSUIBENSDAEMPRESA" => "Possui bens da Empresa (Celular, Notebook e etc)", "VRSALARIO" => "Valor Salário",
        "NROCORRENCIA" => "Nr. Ocorrência", "IDTPSALARIO" => "Tipo de Salário", "NRTPMODALIDSAL" => "Tipo de Modalidade do Salário", "DTALTESALARIO" => "Dt de Alteração Salarial", "DSOBSERVACAO" => "Observação", "DTINIOCUPACAO" => "Dt. Início Sit. Funcional", "NROCUPACAOM" => "Nr. Ocupação",
        "NRTIPOOCUPACAO" => "Nr. Tipo Ocupação", "DTINIESCALA" => "Início da Escala", "NRTURNO" => "Turno", "NRESCALATRABM" => "Escala", "NRTIPORELACIONA" => "Tipo de relacionamento", "NMPESSOADEP" => "Nome", "DTNASCIFUNDPARC" => "Data de nascimento",
        "NRCPFDEP" => "CPF", "NRLINHATRANSP" => "Linha 1", "NRTIPOLINHAVINC" => "Tipo da Linha 1", "VRTARIFA" => "Valor da Tarifa 1", "QTLINHATARIFA" => "Quantidade 1", "NRLINHATRANSP_2" => "Linha 2", "NRTIPOLINHAVINC_2" => "Tipo da Linha 2", "VRTARIFA_2" => "Valor da Tarifa 2", "QTLINHATARIFA_2" => "Quantidade 2", "NRLINHATRANSP_3" => "Linha 3", "NRTIPOLINHAVINC_3" => "Tipo da Linha 3", "VRTARIFA_3" => "Valor da Tarifa 3", "QTLINHATARIFA_3" => "Quantidade 3", "NRLINHATRANSP_4" => "Linha 4", "NRTIPOLINHAVINC_4" => "Tipo da Linha 4", "VRTARIFA_4" => "Valor da Tarifa 4", "QTLINHATARIFA_4" => "Quantidade 4", "NRLINHATRANSP_5" => "Linha 5", "NRTIPOLINHAVINC_5" => "Tipo da Linha 5", "VRTARIFA_5" => "Valor da Tarifa 5", "QTLINHATARIFA_5" => "Quantidade 5", "NRLINHATRANSP_6" => "Linha 6", "NRTIPOLINHAVINC_6" => "Tipo da Linha 6", "VRTARIFA_6" => "Valor da Tarifa 6", "QTLINHATARIFA_6" => "Quantidade 6", "NRLINHATRANSP_7" => "Linha 7", "NRTIPOLINHAVINC_7" => "Tipo da Linha 7", "VRTARIFA_7" => "Valor da Tarifa 7", "QTLINHATARIFA_7" => "Quantidade 7", "NRLINHATRANSP_8" => "Linha 8", "NRTIPOLINHAVINC_8" => "Tipo da Linha 8", "VRTARIFA_8" => "Valor da Tarifa 8", "QTLINHATARIFA_8" => "Quantidade 8", "NRSENTIDOLINHA" => "Sentido da Linha", "DTINILINHAVINC" => "Inicio de Validade", "DTFIMLINHAVINC" => "Fim de Validade", "NRBENEFICIOM" => "Benefício",
        "DTINSCBENEFICIARIO" => "Data Inscrição", "IDTPBENEFICIARIO" => "Tipo Beneficiário", "NRTEMPLATE" => "Template", "NRTIPOPROCESSO" => "Tipo do Processo", "NRPARCNEGOCIORESP" => "Responsável", "DSMOTIVO" => "Motivo", "LOCALIDADE" => "Localidade",
        "DSDESCRICAO" => "Descrição", "DTSOLICITA" => "Disponível desde", "DTLIMSOLICITA" => "Disponível até", "NRESTRUTGEREN" => "Estrutura Gerencial", "NRVAGAS" => "Nº de Vagas", "IDPUBLICAVAGA" => "Publicar Vaga", "NMPROCESSO" => "Nome do Processo",
        "NRTIPOTRANSFER" => "Tipo de Transferência", "NRTPMOVTRANSFM" => "Transferência", "NRMOTIVOTRANSF" => "Motivo Transferência", "NRESTRUTURAM" => "Estrutura", "NRESTRUTLEGAL" => "Estrutura Legal", "DTINIMOVIMENT" => "Data de Início", "DTFIMMOVIMENT" => "Data Final", "DSINICIATIVA" => "Iniciativa", "DSJUSTACAUSA" => "Justa Causa",
        "DSPEREXP" => "Período Experiência", "DSCUMPRIRAVISOPREV" => "Aviso Prévio", "DSCONVENIO" => "Convênio Farmácia (ECX Card)", "DSALTERA" => "Alterar cartão para", "DSNOVAVIA" => "Nova via", "NMTREINAMENTO" => "Nome", "NRCURSO" => "Curso", "NRPERFACESAPROV" => "Perfil de Aprovação",
        "NRMETODOTRE" => "Método", "NRPERFACESEXEC" => "Perfil de Execução", "IDTREINAMENTOINT" => "Treinamento Interno", "DTINIPLANEJADOTRE" => "Início Planejado", "DTFIMPLANEJADOTRE" => "Fim Planejado", "DTAPROVAHE" => "Data da Realização das Horas Extras", "QTINIHOR" => "Hora Inicio", "QTFIMHOR" => "Hora Fim",
        "DSCOMPLCERTNASC" => "Complemento Certidão Nasc.", "NRCERTOBITO" => "Nr. certificado de obito", "NRMOTIVOOCUPA" => "Nr. Motivo Ocupação", "NRTIPOALTERA" => "Tipo de Alteração Salarial", "CDEXCERTRESEPES" => "Cd. Expedição Certificado de Reservista",
        "DSCATEGCERTRESE" => "Categ. Certificado de Reservista", "DTCERTRESEPES" => "Dt. Certificado de Reservista", "NRTITUELEIPES" => "Título de Eleitor", "DSCLASSESTRANG" => "Classificação Trabalhador Estrangeiro", "NRCLASSESTRANG" => "Nr. Classificação Trabalhador Estrangeiro",
        "VRREEMBOLSO" => "Valor do Reembolso", "NRSITUFUNCM" => "Situação Funcional", "DTINISITUFUNC" => "Data Início", "DTFIMSITUFUNC" => "Data Fim", "NRQTDIASITUFUNC" => "Qtde de Dias", "CDTIPOALTEVT" => "Tipo de Alteração", "DTALTELINHAVINC" => "Data de Alteração", "DSOBSLINHAVINC" => "Justificativa da Alteração",
        "DSOBSBENEFICIARIO" => "Justificativa da alteração", "QTBENEFICIO" => "Qtd. Benefício", "DTCANCBENEFICIARIO" => "Data Cancelamento", "CDTIPOALTEVC" => "Tipo de alteração",
        "NREVENTOM" => "Tipo de Evento", "VREVENTOFIXO" => "Valor do Evento", "IDTIPOMOVIMENTO" => "Tipo de Movimento", "DSOBSERVACAOAUXIMORADIA" => "Justificativa da Alteração",
        "NRFERIAS" => "Nr Período Férias", "QTDIASFERIAS" => "Qtd. de dias Férias", "QTDIASABONOFERI" => "Qtd. de dias abono Férias", "DTINIGOZOFERIAS" => "Data Início Férias", "DTFIMGOZOFERIAS" => "Data Fim Férias", "DSGOZOFERIAS" => "Justificativa de férias", "IDADIANT13SALAR" => "Adiantamento de 13º", "CDTIPOVC" => "Tipo", "DSOBSALTESALARIO" => "Justificativa da Alteração", "VRBENEFICIO" => "Vr. Benefício", "DTRESCISAOPROJETADA" => "Dt. Rescisão Projetada", "DSOBSALTEOCUPACAO" => "Justificativa", "DTPREVISTAADMISSAO" => "Data Prevista de Admissão", "NRTIPOVINCULOM" => "Tipo de Vínculo", "NRVINCULOEMPREG" => "Vínculo Empregatício", "NRMOTIVOADMISSAO" => "Motivo da Admissão", "NRPARCNEGOCIOSOLICITA" => "Solicitante", "DSOBSERVACAODESLIGAMENTO" => "Justificativa"
    ];

    private $labelFieldComunica = [
        "PREFTELPRINCIPAL" => "Prefixo Telefone principal", "TELPRINCIPAL" => "Telefone principal", "PREFTELRESIDEN" => "Prefixo Telefone residencial", "TELRESIDEN" => "Telefone residencial",
        "PREFTELCOMERC" => "Prefixo Telefone comercial", "TELCOMERC" => "Telefone comercial", "PREFCELPARTIC" => "Prefixo Celular particular",
        "CELPARTIC" => "Celular particular", "PREFCELCOMERC" => "Prefixo Celular comercial", "CELCOMERC" => "Celular comercial", "PREFFAX" => "Prefixo Fax",
        "FAX" => "Fax", "EMAIL" => "Email", "CDFORMACOMU" => "Cd. Forma Comunicação"
    ];

    private const SOLICITACAO_ALTERACAO_RECEBE_ANEXO = true;

    private const TIPO_SOLICITACAO_CAMPOS = [
        // Férias
        1 => [ 'NRFERIAS', 'QTDIASFERIAS', 'QTDIASABONOFERI', 'DTINIGOZOFERIAS', 'DTFIMGOZOFERIAS', 'DSGOZOFERIAS', 'IDADIANT13SALAR' ],
        // Transferência/Movimentação Legal
        2 => [ 'NRTIPOTRANSFER', 'NRTPMOVTRANSFM', 'NRMOTIVOTRANSF', 'NRESTRUTLEGAL', 'NRESTRUTGEREN', 'DTINIMOVIMENT', 'DTFIMMOVIMENT', 'NRESCALATRABM', 'NRTURNO', 'DTINIESCALA', 'DTALTESALARIO', 'IDTPSALARIO', 'NROCORRENCIA', 'NRTIPOALTERA', 'NRTPMODALIDSAL', 'VRSALARIO', ],
        // Salário
        3 => [ 'DTALTESALARIO', 'IDTPSALARIO', 'NROCORRENCIA', 'NRTIPOALTERA', 'NRTPMODALIDSAL', 'VRSALARIO', 'DSOBSALTESALARIO', ],
        // Treinamento
        4 => [ 'NMTREINAMENTO', 'NRCURSO', 'NRPERFACESAPROV', 'NRMETODOTRE', 'NRPERFACESEXEC', 'IDTREINAMENTOINT', 'DTINIPLANEJADOTRE', 'DTFIMPLANEJADOTRE' ],
        // Desligamento
        5 => [ 'NRTPAVIPRE', 'NRTPDEMISSAO', 'NRMOTIVORESC', 'DTAVISOPREVIO', 'DTRESCISAOVINC', 'DTRESCISAOPROJETADA','CDREPOSICAOVAGA', 'CDPESSOANAOGRATA', 'CDPOSSUIBENSDAEMPRESA', 'DSOBSERVACAODESLIGAMENTO'],
        // Pessoal
        6 => [ 'NRTEMPLATE', 'NRTIPOPROCESSO', 'NRPARCNEGOCIORESP', 'NRPARCNEGOCIOSOLICITA', 'DSMOTIVO', 'LOCALIDADE', 'DSDESCRICAO', 'DTSOLICITA', 'DTLIMSOLICITA', 'NROCUPACAOM', 'NRESTRUTGEREN', 'NRVAGAS', 'IDPUBLICAVAGA', 'NMPROCESSO', ],
        // Meus dados
        7 => [ 'NRCERTCASAPES', 'NRGRAUINSTR', 'DSPESSOA', 'NMPESSOA', 'NRCARTHABPES', 'DSCATEGHABCART', 'DTCASAPESSOA', 'DTHABCARTPES', 'DTVALHABCARTPES', ],
        // Endereço
        8 => [ 'NRCEPENDERECO', 'CDTIPOENDERECO', 'CDPAIS', 'SGESTADO', 'CDMUNICIPIO', 'NMBAIRROENDERECO', 'CDLOGRADOURO', 'DSENDERECO', 'NRIMOVELENDERECO', 'DSCOMPLEENDERECO', 'DSREFERENCIAENDE', ],
        // Movimentação, Ocupação e Salário
        9 => [ 'NROCUPACAOM', 'NRMOTIVOOCUPA', 'NRTPMODALIDSAL', 'NRTIPOOCUPACAO', 'DTINIMOVIMENT', 'DTFIMMOVIMENT', 'IDTPSALARIO', 'NROCORRENCIA', 'NRTIPOALTERA', 'VRSALARIO', 'DSOBSALTESALARIO',  'NRTPMOVTRANSFM', 'NRMOTIVOTRANSF', 'NRESTRUTLEGAL', 'NRESTRUTGEREN'],
        // VT
        10 => [ "NRLINHATRANSP", "NRTIPOLINHAVINC", "VRTARIFA", "QTLINHATARIFA", "NRLINHATRANSP_2", "NRTIPOLINHAVINC_2", "VRTARIFA_2", "QTLINHATARIFA_2", "NRLINHATRANSP_3", "NRTIPOLINHAVINC_3", "VRTARIFA_3", "QTLINHATARIFA_3", "NRLINHATRANSP_4", "NRTIPOLINHAVINC_4", "VRTARIFA_4", "QTLINHATARIFA_4", "NRLINHATRANSP_5", "NRTIPOLINHAVINC_5", "VRTARIFA_5", "QTLINHATARIFA_5", "NRLINHATRANSP_6", "NRTIPOLINHAVINC_6", "VRTARIFA_6", "QTLINHATARIFA_6", "NRLINHATRANSP_7", "NRTIPOLINHAVINC_7", "VRTARIFA_7", "QTLINHATARIFA_7", "NRLINHATRANSP_8", "NRTIPOLINHAVINC_8", "VRTARIFA_8", "QTLINHATARIFA_8", "NRSENTIDOLINHA", "NRSENTIDOLINHA", "DTINILINHAVINC", "DTFIMLINHAVINC", "CDTIPOALTEVT", "DTALTELINHAVINC", "DSOBSLINHAVINC", ],
        // Dependente
        11 => [ "NRTIPORELACIONA", "NMPESSOADEP", "DTNASCIFUNDPARC", "IDSEXOPESSOA", "NRCPFDEP", ],
        // Benefício
        13 => [ "NRBENEFICIOM", "DTINSCBENEFICIARIO", "NRPESSOA", "IDTPBENEFICIARIO", ],
        // Escala
        14 => [ 'NRESCALATRABM', 'NRTURNO', 'DTINIESCALA', ],
        // Conta Corrente
        15 => [ 'CDBANCO', 'CDAGENCIA', 'CDCONTCORR', 'NRTPCONTCORR', ],
        // Aprovacao de Horas Extras
        18 => [ 'DTAPROVAHE', 'QTINIHOR', 'QTFIMHOR', ],
        // Reembolso
        19 => [ 'VRREEMBOLSO', 'DTMESCOMPETENC', ],
        // Advêrtencia/Suspensão
        20 => [ 'NRSITUFUNCM', 'DTINISITUFUNC', 'DTFIMSITUFUNC', 'NRQTDIASITUFUNC', ],
        // Vale Combustível
        21 => [ 'QTBENEFICIO','NRBENEFICIOM', 'VRBENEFICIO', 'DTINSCBENEFICIARIO', 'DTCANCBENEFICIARIO', 'DSOBSBENEFICIARIO', 'CDTIPOALTEVC', 'NRPESSOA', 'CDTIPOVC'],
        // Auxílio Moradia
        22 => [ 'NREVENTOM', 'VREVENTOFIXO', 'IDTIPOMOVIMENTO', 'DSOBSERVACAOAUXIMORADIA', 'DTINIVIGENCIA', 'DTFIMVIGENCIA', ],
        // Vale Manutenção
        23 => [ 'QTBENEFICIO','NRBENEFICIOM', 'DTINSCBENEFICIARIO', 'DTCANCBENEFICIARIO', 'DSOBSBENEFICIARIO', 'CDTIPOALTEVC', 'NRPESSOA'],
        // Banefícios
        24 => [ 'QTBENEFICIO','NRBENEFICIOM', 'DTINSCBENEFICIARIO', 'DTCANCBENEFICIARIO', 'DSOBSBENEFICIARIO', 'CDTIPOALTEVC', 'NRPESSOA'],
        // Substiuição
        25 => [ 'NRTEMPLATE', 'NRTIPOPROCESSO', 'NRPARCNEGOCIORESP', 'NRPARCNEGOCIOSOLICITA', 'DSMOTIVO', 'LOCALIDADE', 'DSDESCRICAO', 'DTSOLICITA', 'DTLIMSOLICITA', 'NROCUPACAOM', 'NRESTRUTGEREN', 'NRVAGAS', 'IDPUBLICAVAGA', 'NMPROCESSO', 'VRSALARIO', 'NRESCALATRABM', 'DTPREVISTAADMISSAO', 'NRTIPOVINCULOM', 'NRVINCULOEMPREG', 'NRGRAUINSTR', 'NRCPFPESSOA', 'NMPESSOA', 'NRMOTIVOADMISSAO'],
        // Transferência/Movimentação Gerencial
        26 => [ 'NRTIPOTRANSFER', 'NRTPMOVTRANSFM', 'NRMOTIVOTRANSF', 'NRESTRUTURAM', 'DTINIMOVIMENT', 'DTFIMMOVIMENT', 'NRESCALATRABM', 'NRTURNO', 'DTINIESCALA', 'DTALTESALARIO', 'IDTPSALARIO', 'NROCORRENCIA', 'NRTIPOALTERA', 'NRTPMODALIDSAL', 'VRSALARIO', ],
        // Ocupação
        27 => [ 'NROCUPACAOM', 'NRMOTIVOOCUPA', 'NRTPMODALIDSAL', 'NRTIPOOCUPACAO', 'DTINIOCUPACAO', 'DSOBSALTEOCUPACAO'],
        // Aumento de Quadro
        28 => [ 'NRTEMPLATE', 'NRTIPOPROCESSO', 'NRPARCNEGOCIORESP', 'NRPARCNEGOCIOSOLICITA', 'DSMOTIVO', 'LOCALIDADE', 'DSDESCRICAO', 'DTSOLICITA', 'DTLIMSOLICITA', 'NROCUPACAOM', 'NRESTRUTGEREN', 'NRVAGAS', 'IDPUBLICAVAGA', 'NMPROCESSO', 'VRSALARIO', 'NRESCALATRABM', 'DTPREVISTAADMISSAO', 'NRTIPOVINCULOM', 'NRVINCULOEMPREG', 'NRGRAUINSTR', 'NRCPFPESSOA', 'NMPESSOA'],
        // Ajuste Salário por Escala
        29 => [ 'DTALTESALARIO', 'IDTPSALARIO', 'NROCORRENCIA', 'NRTIPOALTERA', 'NRTPMODALIDSAL', 'VRSALARIO', 'NRESCALATRABM', 'NRTURNO', 'DTINIESCALA', 'NROCUPACAOM', 'NRMOTIVOOCUPA', 'NRTPMODALIDSAL', 'NRTIPOOCUPACAO', 'DTINIMOVIMENT', 'DTFIMMOVIMENT', 'IDTPSALARIO', 'NROCORRENCIA', 'NRTIPOALTERA', 'VRSALARIO', 'DSOBSALTESALARIO',  'NRTPMOVTRANSFM', 'NRMOTIVOTRANSF', 'NRESTRUTLEGAL', 'NRESTRUTGEREN' ],
    ];
    
    const TIPO_SOLICITACAO_CAMPOS_ONLY_EMAIL = [
        // Vale alimentacao/refeicao
        -3 => [ 'DSALTERA' ],
        // Nova via cartao
        -2 => [ 'DSNOVAVIA', 'DSMOTIVO' ],
        // Convenio farmacia
        -1 => [ 'DSCONVENIO' ],
        // Desligamento
        5 => [ 'DSINICIATIVA','DSJUSTACAUSA', 'DSPEREXP', 'DSCUMPRIRAVISOPREV', ],
    ];
    
    const TIPO_SOLICITACAO_OBSERVACOES_FINAIS = [
        // Nova via cartao
        -2 => EmailMessages::SOLICITACAO_CARTAO_OBSERVACOES_CONTENT,
    ];

    const CDFORMACOMU_TELEFONE_PRINCIPAL    = '01';
    const CDFORMACOMU_CELULAR_PARTICULAR    = '02';
    const CDFORMACOMU_TELEFONE_RESIDENCIAL  = '03';
    const CDFORMACOMU_TELEFONE_COMERCIAL    = '04';
    const CDFORMACOMU_EMAIL                 = '05';
    const CDFORMACOMU_CELULAR_COMERCIAL     = '06';
    const CDFORMACOMU_FAX                   = '10';

    public function __construct(
        MinhaHistoriaRepository $minhaHistoriaRepository,
        VinculoRepository $vinculoRepository,
        SolicitacaoRepository $solicitacaoRepository,
        Environment $environment,
        EmailService $emailService,
        Utilities $utilities
    ) {
        $this->minhaHistoriaRepository = $minhaHistoriaRepository;
        $this->vinculoRepository       = $vinculoRepository;
        $this->solicitacaoRepository   = $solicitacaoRepository;
        $this->environment             = $environment;
        $this->emailService            = $emailService;
        $this->utilities               = $utilities;
    }

    public function solicitaAlteracao($row): ?string {
        try {
            $nrorg      = $row['NRORG'];
            $nrvinculom = $row['NRVINCULOM'];
            $tipoSoli   = $row['NRTIPOSOLICITACAO'];
            $filePath   = null;
            
            // Anexo ao email
            if (isset($row['DSFILE']) && isset($row['NMFILE'])) {
                $fileContent    = $row['DSFILE'];
                $fileName       = $row['NMFILE'];

                $row['DSPATH'] = $this->saveFile($row);

                unset($row['DSFILE']);
                unset($row['NMFILE']);
                
                $filePath = Utilities::blobToFile($fileContent, $fileName);
            }
            
            if (isset($row['DSOBSSOLICITACAO'])) {
                $dsobservacao = $row['DSOBSSOLICITACAO'];
            } else {
                $dsobservacao = null;
            }

            $cdoperador = null;

            if (isset($row['CDOPERADOR'])) {
                $cdoperador = (string) $row['CDOPERADOR'];
                unset($row['CDOPERADOR']);
            } else {
                $cdoperador = $this->vinculoRepository->getOperadorVinculo($nrorg, $nrvinculom);
                $cdoperador = $cdoperador['CDOPERADOR'];
            }
            
            $onlyInEmailItens = $this->getOnlyEmailData($tipoSoli, $row);
            
            if (SolicitacaoService::isTipoSoliOnlyEmail($tipoSoli)) {
                $this->emailService->notificaMinhaHistoria(
                    $nrvinculom,
                    null,
                    $tipoSoli,
                    $onlyInEmailItens,
                    $filePath
                );
                
                return null;
            }

            $nrsolicitacao = NovoCodigo::geraCodigo($nrorg, 'GPE_SOLICITACAO', 1, 12, true);
            $itens = $this->validateTipoSolicitacaoCampos($tipoSoli, $row);
            $file = $row['DSPATH'] ?? null;


            $this->minhaHistoriaRepository->insereSolicitacao($nrsolicitacao, $tipoSoli, $nrvinculom, $nrorg, $cdoperador, $dsobservacao, $file);

            foreach ($itens as $key => $field) {
                if(!in_array($key, $this->default_key) && !empty($field)){
                    $nritemsolicitacao = NovoCodigo::geraCodigo($nrorg, 'GPE_ITEMSOLICITACAO', 1, 12, true);
                    $this->minhaHistoriaRepository->insereItemSolicitacao($nritemsolicitacao, $nrsolicitacao, $key, $this->labelFieldPessoa[$key], strval($field), $nrorg, $cdoperador);
                }
            }
            // $this->emailService->notificaMinhaHistoria($nrvinculom, $nrsolicitacao, $filePath);
            $this->emailService->notificaMinhaHistoria(
                $nrvinculom,
                $nrsolicitacao,
                null,
                $onlyInEmailItens,
                $filePath
            );
            return $nrsolicitacao;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getSolicitacaoFile(
        int $nrSolicitacao
    ): array {
        $nrOrg = (int) $this->environment->getNrOrg();

        $solicitacao = $this->solicitacaoRepository->getSolicitacao(
            $nrOrg, $nrSolicitacao
        );

        if (!$solicitacao) {
            return '';
        }

        $path = isset($solicitacao['DSPATH']) ?
            $solicitacao['DSPATH'] :
            '';

        if ($path === '') {
            return '';
        }

        $file = $this->utilities->downloadFile($path);

        return $file;
    }

    public function solicitaAlteracaoComunica($row): ?string {
        $nrorg      = $row['NRORG'];
        $nrvinculom = $row['NRVINCULOM'];
        $tipoSoli   = $row['NRTIPOSOLICITACAO'];
        $filePath   = null;
        if(isset($row['DSOBSSOLICITACAO']))
            $dsobservacao = $row['DSOBSSOLICITACAO'];
        else
            $dsobservacao = null;

        $cdoperador = $this->vinculoRepository->getOperadorVinculo($nrorg, $nrvinculom);
        $cdoperador = $cdoperador['CDOPERADOR'];

        if (isset($row['DSFILE']) && isset($row['NMFILE'])) {
            $fileContent    = $row['DSFILE'];
            $fileName       = $row['NMFILE'];
            unset($row['DSFILE']);
            unset($row['NMFILE']);

            $filePath = Utilities::blobToFile($fileContent, $fileName);
        }

        if(isset($row['PREFTELPRINCIPAL']) || isset($row['TELPRINCIPAL'])){
            $nrsolicitacao = NovoCodigo::geraCodigo($nrorg, 'GPE_SOLICITACAO', 1, 12, true);
            $this->minhaHistoriaRepository->insereSolicitacao($nrsolicitacao, $tipoSoli, $nrvinculom, $nrorg, $cdoperador, $dsobservacao);
            if(isset($row['PREFTELPRINCIPAL'])){
                $nritemsolicitacao = NovoCodigo::geraCodigo($nrorg, 'GPE_ITEMSOLICITACAO', 1, 12, true);
                $this->minhaHistoriaRepository->insereItemSolicitacao($nritemsolicitacao, $nrsolicitacao, 'CDPREFIXCOMUPARC', $this->labelFieldComunica['PREFTELPRINCIPAL'], $row['PREFTELPRINCIPAL'], $nrorg, $cdoperador);
            }
            if(isset($row['TELPRINCIPAL'])){
                $nritemsolicitacao = NovoCodigo::geraCodigo($nrorg, 'GPE_ITEMSOLICITACAO', 1, 12, true);
                $this->minhaHistoriaRepository->insereItemSolicitacao($nritemsolicitacao, $nrsolicitacao, 'CDCOMUNICAPARC', $this->labelFieldComunica['TELPRINCIPAL'], $row['TELPRINCIPAL'], $nrorg, $cdoperador);
            }

            $nritemsolicitacao = NovoCodigo::geraCodigo($nrorg, 'GPE_ITEMSOLICITACAO', 1, 12, true);
            $this->minhaHistoriaRepository->insereItemSolicitacao($nritemsolicitacao, $nrsolicitacao, 'CDFORMACOMU', $this->labelFieldComunica['CDFORMACOMU'], self::CDFORMACOMU_TELEFONE_PRINCIPAL, $nrorg, $cdoperador);

            $this->emailService->notificaMinhaHistoria(
                $nrvinculom,
                $nrsolicitacao,
                $tipoSoli,
                [],
                $filePath
            );
        }

        if(isset($row['PREFTELRESIDEN']) || isset($row['TELRESIDEN'])){
            $nrsolicitacao = NovoCodigo::geraCodigo($nrorg, 'GPE_SOLICITACAO', 1, 12, true);
            $this->minhaHistoriaRepository->insereSolicitacao($nrsolicitacao, $tipoSoli, $nrvinculom, $nrorg, $cdoperador, $dsobservacao);
            if(isset($row['PREFTELRESIDEN'])){
                $nritemsolicitacao = NovoCodigo::geraCodigo($nrorg, 'GPE_ITEMSOLICITACAO', 1, 12, true);
                $this->minhaHistoriaRepository->insereItemSolicitacao($nritemsolicitacao, $nrsolicitacao, 'CDPREFIXCOMUPARC', $this->labelFieldComunica['PREFTELRESIDEN'], $row['PREFTELRESIDEN'], $nrorg, $cdoperador);
            }
            if(isset($row['TELRESIDEN'])){
                $nritemsolicitacao = NovoCodigo::geraCodigo($nrorg, 'GPE_ITEMSOLICITACAO', 1, 12, true);
                $this->minhaHistoriaRepository->insereItemSolicitacao($nritemsolicitacao, $nrsolicitacao, 'CDCOMUNICAPARC', $this->labelFieldComunica['TELRESIDEN'], $row['TELRESIDEN'], $nrorg, $cdoperador);
            }

            $nritemsolicitacao = NovoCodigo::geraCodigo($nrorg, 'GPE_ITEMSOLICITACAO', 1, 12, true);
            $this->minhaHistoriaRepository->insereItemSolicitacao($nritemsolicitacao, $nrsolicitacao, 'CDFORMACOMU', $this->labelFieldComunica['CDFORMACOMU'], self::CDFORMACOMU_TELEFONE_RESIDENCIAL, $nrorg, $cdoperador);

            $this->emailService->notificaMinhaHistoria(
                $nrvinculom,
                $nrsolicitacao,
                $tipoSoli,
                [],
                $filePath
            );
        }

        if(isset($row['PREFTELCOMERC']) || isset($row['TELCOMERC'])){
            $nrsolicitacao = NovoCodigo::geraCodigo($nrorg, 'GPE_SOLICITACAO', 1, 12, true);
            $this->minhaHistoriaRepository->insereSolicitacao($nrsolicitacao, $tipoSoli, $nrvinculom, $nrorg, $cdoperador, $dsobservacao);
            if(isset($row['PREFTELCOMERC'])){
                $nritemsolicitacao = NovoCodigo::geraCodigo($nrorg, 'GPE_ITEMSOLICITACAO', 1, 12, true);
                $this->minhaHistoriaRepository->insereItemSolicitacao($nritemsolicitacao, $nrsolicitacao, 'CDPREFIXCOMUPARC', $this->labelFieldComunica['PREFTELCOMERC'], $row['PREFTELCOMERC'], $nrorg, $cdoperador);
            }
            if(isset($row['TELCOMERC'])){
                $nritemsolicitacao = NovoCodigo::geraCodigo($nrorg, 'GPE_ITEMSOLICITACAO', 1, 12, true);
                $this->minhaHistoriaRepository->insereItemSolicitacao($nritemsolicitacao, $nrsolicitacao, 'CDCOMUNICAPARC', $this->labelFieldComunica['TELCOMERC'], $row['TELCOMERC'], $nrorg, $cdoperador);
            }

            $nritemsolicitacao = NovoCodigo::geraCodigo($nrorg, 'GPE_ITEMSOLICITACAO', 1, 12, true);
            $this->minhaHistoriaRepository->insereItemSolicitacao($nritemsolicitacao, $nrsolicitacao, 'CDFORMACOMU', $this->labelFieldComunica['CDFORMACOMU'], self::CDFORMACOMU_TELEFONE_COMERCIAL, $nrorg, $cdoperador);

            $this->emailService->notificaMinhaHistoria(
                $nrvinculom,
                $nrsolicitacao,
                $tipoSoli,
                [],
                $filePath
            );
        }

        if(isset($row['PREFCELPARTIC']) || isset($row['CELPARTIC'])){
            $nrsolicitacao = NovoCodigo::geraCodigo($nrorg, 'GPE_SOLICITACAO', 1, 12, true);
            $this->minhaHistoriaRepository->insereSolicitacao($nrsolicitacao, $tipoSoli, $nrvinculom, $nrorg, $cdoperador, $dsobservacao);
            if(isset($row['PREFCELPARTIC'])){
                $nritemsolicitacao = NovoCodigo::geraCodigo($nrorg, 'GPE_ITEMSOLICITACAO', 1, 12, true);
                $this->minhaHistoriaRepository->insereItemSolicitacao($nritemsolicitacao, $nrsolicitacao, 'CDPREFIXCOMUPARC', $this->labelFieldComunica['PREFCELPARTIC'], $row['PREFCELPARTIC'], $nrorg, $cdoperador);
            }
            if(isset($row['CELPARTIC'])){
                $nritemsolicitacao = NovoCodigo::geraCodigo($nrorg, 'GPE_ITEMSOLICITACAO', 1, 12, true);
                $this->minhaHistoriaRepository->insereItemSolicitacao($nritemsolicitacao, $nrsolicitacao, 'CDCOMUNICAPARC', $this->labelFieldComunica['CELPARTIC'], $row['CELPARTIC'], $nrorg, $cdoperador);
            }

            $nritemsolicitacao = NovoCodigo::geraCodigo($nrorg, 'GPE_ITEMSOLICITACAO', 1, 12, true);
            $this->minhaHistoriaRepository->insereItemSolicitacao($nritemsolicitacao, $nrsolicitacao, 'CDFORMACOMU', $this->labelFieldComunica['CDFORMACOMU'], self::CDFORMACOMU_CELULAR_PARTICULAR, $nrorg, $cdoperador);

            $this->emailService->notificaMinhaHistoria(
                $nrvinculom,
                $nrsolicitacao,
                $tipoSoli,
                [],
                $filePath
            );
        }

        if(isset($row['PREFCELCOMERC']) || isset($row['CELCOMERC'])){
            $nrsolicitacao = NovoCodigo::geraCodigo($nrorg, 'GPE_SOLICITACAO', 1, 12, true);
            $this->minhaHistoriaRepository->insereSolicitacao($nrsolicitacao, $tipoSoli, $nrvinculom, $nrorg, $cdoperador, $dsobservacao);
            if(isset($row['PREFCELCOMERC'])){
                $nritemsolicitacao = NovoCodigo::geraCodigo($nrorg, 'GPE_ITEMSOLICITACAO', 1, 12, true);
                $this->minhaHistoriaRepository->insereItemSolicitacao($nritemsolicitacao, $nrsolicitacao, 'CDPREFIXCOMUPARC', $this->labelFieldComunica['PREFCELCOMERC'], $row['PREFCELCOMERC'], $nrorg, $cdoperador);
            }
            if(isset($row['CELCOMERC'])){
                $nritemsolicitacao = NovoCodigo::geraCodigo($nrorg, 'GPE_ITEMSOLICITACAO', 1, 12, true);
                $this->minhaHistoriaRepository->insereItemSolicitacao($nritemsolicitacao, $nrsolicitacao, 'CDCOMUNICAPARC', $this->labelFieldComunica['CELCOMERC'], $row['CELCOMERC'], $nrorg, $cdoperador);
            }

            $nritemsolicitacao = NovoCodigo::geraCodigo($nrorg, 'GPE_ITEMSOLICITACAO', 1, 12, true);
            $this->minhaHistoriaRepository->insereItemSolicitacao($nritemsolicitacao, $nrsolicitacao, 'CDFORMACOMU', $this->labelFieldComunica['CDFORMACOMU'], self::CDFORMACOMU_CELULAR_COMERCIAL, $nrorg, $cdoperador);

            $this->emailService->notificaMinhaHistoria(
                $nrvinculom,
                $nrsolicitacao,
                $tipoSoli,
                [],
                $filePath
            );
        }

        if(isset($row['PREFFAX']) || isset($row['FAX'])){
            $nrsolicitacao = NovoCodigo::geraCodigo($nrorg, 'GPE_SOLICITACAO', 1, 12, true);
            $this->minhaHistoriaRepository->insereSolicitacao($nrsolicitacao, $tipoSoli, $nrvinculom, $nrorg, $cdoperador, $dsobservacao);
            if(isset($row['PREFFAX'])){
                $nritemsolicitacao = NovoCodigo::geraCodigo($nrorg, 'GPE_ITEMSOLICITACAO', 1, 12, true);
                $this->minhaHistoriaRepository->insereItemSolicitacao($nritemsolicitacao, $nrsolicitacao, 'CDPREFIXCOMUPARC', $this->labelFieldComunica['PREFFAX'], $row['PREFFAX'], $nrorg, $cdoperador);
            }
            if(isset($row['FAX'])){
                $nritemsolicitacao = NovoCodigo::geraCodigo($nrorg, 'GPE_ITEMSOLICITACAO', 1, 12, true);
                $this->minhaHistoriaRepository->insereItemSolicitacao($nritemsolicitacao, $nrsolicitacao, 'CDCOMUNICAPARC', $this->labelFieldComunica['FAX'], $row['FAX'], $nrorg, $cdoperador);
            }

            $nritemsolicitacao = NovoCodigo::geraCodigo($nrorg, 'GPE_ITEMSOLICITACAO', 1, 12, true);
            $this->minhaHistoriaRepository->insereItemSolicitacao($nritemsolicitacao, $nrsolicitacao, 'CDFORMACOMU', $this->labelFieldComunica['CDFORMACOMU'], self::CDFORMACOMU_FAX, $nrorg, $cdoperador);

            $this->emailService->notificaMinhaHistoria(
                $nrvinculom,
                $nrsolicitacao,
                $tipoSoli,
                [],
                $filePath
            );
        }

        if(isset($row['EMAIL']) ){
            $nrsolicitacao = NovoCodigo::geraCodigo($nrorg, 'GPE_SOLICITACAO', 1, 12, true);
            $this->minhaHistoriaRepository->insereSolicitacao($nrsolicitacao, $tipoSoli, $nrvinculom, $nrorg, $cdoperador, $dsobservacao);
            $nritemsolicitacao = NovoCodigo::geraCodigo($nrorg, 'GPE_ITEMSOLICITACAO', 1, 12, true);
            $this->minhaHistoriaRepository->insereItemSolicitacao($nritemsolicitacao, $nrsolicitacao, 'CDCOMUNICAPARC', $this->labelFieldComunica['EMAIL'], $row['EMAIL'], $nrorg, $cdoperador);

            $nritemsolicitacao = NovoCodigo::geraCodigo($nrorg, 'GPE_ITEMSOLICITACAO', 1, 12, true);
            $this->minhaHistoriaRepository->insereItemSolicitacao($nritemsolicitacao, $nrsolicitacao, 'CDFORMACOMU', $this->labelFieldComunica['CDFORMACOMU'], self::CDFORMACOMU_EMAIL, $nrorg, $cdoperador);

            $this->emailService->notificaMinhaHistoria(
                $nrvinculom,
                $nrsolicitacao,
                $tipoSoli,
                [],
                $filePath
            );
        }

        return $nrsolicitacao;
    }

    /**
     *  Rotina responsável por filtrar (sanitize) os campos necessários para a criação de cada solicitação, por tipo. Campos com valor nulo não são retornados,
     * visto que a coluna de valor dos itens na tabela de itens não pode receber nulo
     *
     *  @param  string      $tipoSoli
     *  @param  Array/Row   $itensSolicitacao
     *
     *  @return Array       $itensSanitized
     *
     */
    public function validateTipoSolicitacaoCampos($tipoSoli, $itensSolicitacao) {
        $camposItem = isset(self::TIPO_SOLICITACAO_CAMPOS[$tipoSoli]) ? self::TIPO_SOLICITACAO_CAMPOS[$tipoSoli] : false;
        if (empty($camposItem)) {
            return [];
        }

        $itens = !is_array($itensSolicitacao) ? (array) $itensSolicitacao : $itensSolicitacao;
        $itensSanitized = array_filter($itens, function($itemKey) use ($camposItem, $itens) {
            return in_array($itemKey, $camposItem) && !is_null($itens[$itemKey]);
        }, ARRAY_FILTER_USE_KEY);
        
        return $itensSanitized;
    }
    
    /**
     *  Rotina responsável por filtrar (sanitize) os campos que serao enviados por email somente
     *
     *  @param  string      $tipoSoli
     *  @param  Array/Row   $itensSolicitacao
     *
     *  @return Array       $itensSanitized
     *
     */
    public function getOnlyEmailData(string $tipoSoli, $itensSolicitacao) {
        $itens = !is_array($itensSolicitacao) ? (array) $itensSolicitacao : $itensSolicitacao;
        $itensSanitized = array_filter($itens, function($itemKey) use ($itens, $tipoSoli) {
            return !empty(self::TIPO_SOLICITACAO_CAMPOS_ONLY_EMAIL[$tipoSoli]) && in_array($itemKey, self::TIPO_SOLICITACAO_CAMPOS_ONLY_EMAIL[$tipoSoli]) && !is_null($itens[$itemKey]);
        }, ARRAY_FILTER_USE_KEY);
        
        $itensDefaultFormat = [];
        foreach ($itensSanitized as $key => $item) {
            $itensDefaultFormat[] = [
                'DSLABELITEMSOLI' => $this->labelFieldPessoa[$key] ?? $key,
                'DSVALORITEMSOLI' => is_array($item) ? implode(', ', $item) : $item,
            ];
        }

        return $itensDefaultFormat;
    }

    public function saveFile($row) {
        $nrOrg = (int) $this->environment->getNrOrg();

        if (isset($row['DSFILE']) && $row['DSFILE'] !== '') {
            $file = $row['DSFILE'];

            $filePath = $this->utilities->saveFileOnFileServer(
                $file, 'jpeg', $nrOrg, null, null, '', false
            );

            return $filePath;
        }

        return NULL;
    }

    /**
     *  Rotina responsável por verificar se já existe uma solicitação igual para o dia em que está sendo realizado a solicitação
     *
     *  @param  Array/Row   $itensSolicitacao
     *
     *  @return Bool/Array  false/$nrSolicitacao
     *
     */
    public function verificaBloqueioSolicitacao($row) {
        $dslabelitemsoli = '';
        $dsvaloritemsoli = '';
        $tabela          = '';
        $campoData       = '';
        $campoAlteracao  = '';
        $nrorg           = $row['NRORG'];
        $nrvinculom      = $row['NRVINCULOM'];
        $tipoSoli        = $row['NRTIPOSOLICITACAO'];
        $dtatual         = DateUtil::getDataDeString(substr(date('d/m/Y'),0,10), DateUtil::FORMATO_BRASILEIRO, true)->format('d/m/Y');

        switch($tipoSoli) {
            case '3':
                $dslabelitemsoli = 'Dt de Alteração Salarial';
                $dsvaloritemsoli = $row['DTALTESALARIO'];
                $tabela          = 'GPE_ALTESALARIO';
                $campoData       = 'DTALTESALARIO';
                $campoAlteracao  = 'NRALTESALARIO';
                break;
            case '9':
                $dslabelitemsoli = 'Dt. Início Sit. Funcional';
                $dsvaloritemsoli = $row['DTINIOCUPACAO'];
                $tabela          = 'GPE_ALTEOCUPACAO';
                $campoData       = 'DTINIOCUPACAO';
                $campoAlteracao  = 'NRALTEOCUPACAO';
                break;
            case '21':
                $dslabelitemsoli = 'Data Inscrição';
                $dsvaloritemsoli = $row['DTINSCBENEFICIARIO'];
                break;
            case '22':
                $dslabelitemsoli = 'Início Vigência Conselho';
                $dsvaloritemsoli = $row['DTINIVIGENCIA'];
                break;
            case '27':
                $dslabelitemsoli = 'Dt. Início Sit. Funcional';
                $dsvaloritemsoli = $row['DTINIOCUPACAO'];
                $tabela          = 'GPE_ALTEOCUPACAO';
                $campoData       = 'DTINIOCUPACAO';
                $campoAlteracao  = 'NRALTEOCUPACAO';
                break;
            case '29':
                $dslabelitemsoli = 'Dt de Alteração Salarial';
                $dsvaloritemsoli = $row['DTALTESALARIO'];
                $tabela          = 'GPE_ALTESALARIO';
                $campoData       = 'DTALTESALARIO';
                $campoAlteracao  = 'NRALTESALARIO';
                break;
            default:
                $dslabelitemsoli = '';
                $dsvaloritemsoli = '';
                break;
        }

        if($dslabelitemsoli == '' || $dsvaloritemsoli == '') {
            return false;
        }

        $params = array(
            'P_NRVINCULOM'         => $nrvinculom,
            'P_NRORG'              => $nrorg,
            'P_DSLABELITEMSOLI'    => $dslabelitemsoli,
            'P_DSVALORITEMSOLI'    => $dsvaloritemsoli,
            'P_NRTIPOSOLICITACAO'  => $row["NRTIPOSOLICITACAO"],
        );
        
        $result = $this->minhaHistoriaRepository->verificaBloqueioSolicitacao($params);
        
        // Já retorna o erro caso exista uma solicitação com o mesmo dia e tipo
        if(!empty($result)) {
            return $result;
        }

        // Valida se tem uma alteração de mesmo tipo para o mesmo dia para o vínculo
        if($tipoSoli == '22') { // EVENTOS
        } 
        else if($tipoSoli == '21') { // BENEFICIOS
            $nrbeneficio = $row['NRBENEFICIOM'];
            $params = array(
                'P_NRORG'          => $nrorg,
                'P_NRVINCULOM'     => $nrvinculom,
                'P_NRBENEFICIOM'   => $nrbeneficio,
                'P_DTATUAl'        => $dtatual
            );
            
            $resultAlteracao = $this->minhaHistoriaRepository->verificaBeneficiario($params);
        } 
        else {
            $paramsAlteracao = array(
                "P_NRORG"           => $nrorg,
                "P_NRVINCULOM"      => $nrvinculom,
                "P_CAMPODATA"       => $dsvaloritemsoli
            );
            $resultAlteracao = $this->minhaHistoriaRepository->verificaBloqueioAlteracao($tabela, $campoAlteracao, $campoData, $paramsAlteracao);
        }

        /* Caso seja alteração de ocupação e salário, deve-se olhar para as alterações de ambas tabelas
        *  Caso uma alteração seja localizada na alteração de ocupação, é retornado o n° de alteração
        *  Caso contrário, é pesquisado na alteração de salário e caso encontrado, retornado seu n°
        */
        if($tipoSoli == '9' && empty($resultAlteracao)) {
            $resultAlteracao = $this->verificaBloqueioSolicitacaoOcupacaoSalario($row);
        }
        if($tipoSoli == '29' && empty($resultAlteracao)) {
            $resultAlteracao = $this->verificaBloqueioSolicitacaoSalarioEscala($row);
        }
        
        return !empty($resultAlteracao) ? $resultAlteracao : false;
    }

    public function verificaBloqueioSolicitacaoOcupacaoSalario($row) {
        $nrorg = $row['NRORG'];
        $nrvinculom = $row['NRVINCULOM'];
        $dslabelitemsoli = 'Dt de Alteração de Ocupação';
        $dsvaloritemsoli = $row['DTINIOCUPACAO'];
        $tabela          = 'GPE_ALTEOCUPACAO';
        $campoData       = 'DTINIOCUPACAO';
        $campoAlteracao  = 'NRALTEOCUPACAO';

        $params = array(
            'P_NRVINCULOM'         => $nrvinculom,
            'P_NRORG'              => $nrorg,
            'P_DSLABELITEMSOLI'    => $dslabelitemsoli,
            'P_DSVALORITEMSOLI'    => $dsvaloritemsoli,
            'P_NRTIPOSOLICITACAO'  => $row["NRTIPOSOLICITACAO"],
        );
        
        $resultAlteracao = $this->minhaHistoriaRepository->verificaBloqueioSolicitacao($params);

        // Já retorna o erro caso exista uma solicitação com o mesmo dia e tipo
        if(!empty($resultAlteracao)) {
            return $resultAlteracao;
        }

        // Valida se tem uma alteração de mesmo tipo para o mesmo dia para o vínculo
        $paramsAlteracao = array(
            "P_NRORG"           => $nrorg,
            "P_NRVINCULOM"      => $nrvinculom,
            "P_CAMPODATA"       => $dsvaloritemsoli
        );
        $resultAlteracao = $this->minhaHistoriaRepository->verificaBloqueioAlteracao($tabela, $campoAlteracao, $campoData, $paramsAlteracao);

        /* Caso seja alteração de ocupação e salário, deve-se olhar para as alterações de ambas tabelas
        *  Caso uma alteração seja localizada na alteração de ocupação, é retornado o n° de alteração
        *  Caso contrário, é pesquisado na alteração de salário e caso encontrado, retornado seu n°
        */
        if(empty($resultAlteracao) && !empty($row['VRSALARIO'])) {
            $dslabelitemsoli = 'Dt de Alteração Salarial';
            $dsvaloritemsoli = $row['DTINIOCUPACAO'];

            $paramsAlteracao = array(
                "P_NRORG"           => $nrorg,
                "P_NRVINCULOM"      => $nrvinculom,
                "P_CAMPODATA"       => $dsvaloritemsoli
            );

            $resultAlteracao = $this->minhaHistoriaRepository->verificaBloqueioAlteracao('GPE_ALTESALARIO', 'NRALTESALARIO', 'DTALTESALARIO', $paramsAlteracao);
        }

        // Caso seja alteração de ocupação e salário, deve-se olhar para as solicitação de ambas 
        if(empty($resultAlteracao)) {
            $params = array(
                'P_NRVINCULOM'         => $nrvinculom,
                'P_NRORG'              => $nrorg,
                'P_DSLABELITEMSOLI'    => 'Dt. Início Sit. Funcional',
                'P_DSVALORITEMSOLI'    => $dsvaloritemsoli,
                'P_NRTIPOSOLICITACAO'  => "27",
            );

            $resultAlteracao = $this->minhaHistoriaRepository->verificaBloqueioSolicitacao($params);

            if(empty($resultAlteracao) && !empty($row['VRSALARIO'])) {
                $params = array(
                    'P_NRVINCULOM'         => $nrvinculom,
                    'P_NRORG'              => $nrorg,
                    'P_DSLABELITEMSOLI'    => 'Dt de Alteração Salarial',
                    'P_DSVALORITEMSOLI'    => $dsvaloritemsoli,
                    'P_NRTIPOSOLICITACAO'  => "3",
                );
    
                $resultAlteracao = $this->minhaHistoriaRepository->verificaBloqueioSolicitacao($params);
            }
        }
        
        return !empty($resultAlteracao) ? $resultAlteracao : false;
    }

    public function verificaBloqueioSolicitacaoSalarioEscala($row) {
        $nrorg = $row['NRORG'];
        $nrvinculom = $row['NRVINCULOM'];
        $dslabelitemsoli = 'Dt de Alteração Salarial';
        $dsvaloritemsoli = $row['DTALTESALARIO'];
        $tabela          = 'GPE_ALTESALARIO';
        $campoData       = 'DTALTESALARIO';
        $campoAlteracao  = 'NRALTESALARIO';

        $params = array(
            'P_NRVINCULOM'         => $nrvinculom,
            'P_NRORG'              => $nrorg,
            'P_DSLABELITEMSOLI'    => $dslabelitemsoli,
            'P_DSVALORITEMSOLI'    => $dsvaloritemsoli,
            'P_NRTIPOSOLICITACAO'  => $row["NRTIPOSOLICITACAO"],
        );
        
        $resultAlteracao = $this->minhaHistoriaRepository->verificaBloqueioSolicitacao($params);

        // Já retorna o erro caso exista uma solicitação com o mesmo dia e tipo
        if(!empty($resultAlteracao)) {
            return $resultAlteracao;
        }

        // Valida se tem uma alteração de mesmo tipo para o mesmo dia para o vínculo
        $paramsAlteracao = array(
            "P_NRORG"           => $nrorg,
            "P_NRVINCULOM"      => $nrvinculom,
            "P_CAMPODATA"       => $dsvaloritemsoli
        );
        $resultAlteracao = $this->minhaHistoriaRepository->verificaBloqueioAlteracao($tabela, $campoAlteracao, $campoData, $paramsAlteracao);


        /* Caso seja alteração de ocupação e salário, deve-se olhar para as alterações de ambas tabelas
        *  Caso uma alteração seja localizada na alteração de ocupação, é retornado o n° de alteração
        *  Caso contrário, é pesquisado na alteração de salário e caso encontrado, retornado seu n°
        */
        if(empty($resultAlteracao) && !empty($row['NROCUPACAOM'])) {
            $dslabelitemsoli = 'Dt de Alteração de Ocupação';
            $dsvaloritemsoli = $row['DTINIOCUPACAO'];

            $paramsAlteracao = array(
                "P_NRORG"           => $nrorg,
                "P_NRVINCULOM"      => $nrvinculom,
                "P_CAMPODATA"       => $dsvaloritemsoli
            );

            $resultAlteracao = $this->minhaHistoriaRepository->verificaBloqueioAlteracao('GPE_ALTEOCUPACAO', 'NRALTEOCUPACAO', 'DTINIOCUPACAO', $paramsAlteracao);
        }

        // Caso seja alteração de ocupação e salário, deve-se olhar para as solicitação de ambas 
        if(empty($resultAlteracao) && !empty($row['NROCUPACAOM'])) {
            $params = array(
                'P_NRVINCULOM'         => $nrvinculom,
                'P_NRORG'              => $nrorg,
                'P_DSLABELITEMSOLI'    => 'Dt. Início Sit. Funcional',
                'P_DSVALORITEMSOLI'    => $dsvaloritemsoli,
                'P_NRTIPOSOLICITACAO'  => "27",
            );

            $resultAlteracao = $this->minhaHistoriaRepository->verificaBloqueioSolicitacao($params);

            if(empty($resultAlteracao) && !empty($row['VRSALARIO'])) {
                $params = array(
                    'P_NRVINCULOM'         => $nrvinculom,
                    'P_NRORG'              => $nrorg,
                    'P_DSLABELITEMSOLI'    => 'Dt de Alteração Salarial',
                    'P_DSVALORITEMSOLI'    => $dsvaloritemsoli,
                    'P_NRTIPOSOLICITACAO'  => "3",
                );
    
                $resultAlteracao = $this->minhaHistoriaRepository->verificaBloqueioSolicitacao($params);
            }
        }
        
        return !empty($resultAlteracao) ? $resultAlteracao : false;
    }
}
