<?php

namespace Ems\Util;

use Util\Environment;
use Ems\Util\FileServerClasses\FSMigration\FileServer\Files\FSFilesClient;
use Ems\Repository\OrganizacaoRepository;

class FileServer {
    /** @var FSFilesClient */
    protected $client;

    /** @var OrganizacaoRepository */
    protected $organizacaoRepository;

    public function __construct(
        Environment $environment,
        string $baseUrl,
        OrganizacaoRepository $organizacaoRepository
    ) {
        $nrOrg = $environment->getNrOrg();
        $key = $this->getAPIKey($nrOrg, $organizacaoRepository);

        $this->client = new FSFilesClient($nrOrg, $key, $baseUrl);
    }

    public static function getAPIKey(
        int $nrOrg,
        OrganizacaoRepository $organizacaoRepository
    ): string {
        $key = '';

        if (isset($_SESSION['FSKEY'])) {
            $key = $_SESSION['FSKEY'];
        } else {
            $organizacao = $organizacaoRepository->getOrganizacao($nrOrg, true);
            $key = $organizacao['DSFSAPIKEY'] ?? '';
            $_SESSION['FSKEY'] = $key;
        }

        if ($key === '') {
            $message = "A organização $nrOrg não possui uma API Key do File Server!";
            throw new \Exception($message);
        }

        return $key;
    }

    public function upload($params) {
        $path = $this->client->uploadFile($params);

        return $path;
    }

    /**
    * Requests the FileServer the file with the respective path.
    * @param string $path A string containg the path to the file once stored at FileServer.
    * @return array Array with the file in base64 format and its MIME type.
    */
    public function download(string $path): array {
        $file = $this->client->getFile($path);

        return $file;
    }

    public function delete(string $path): bool {
        $deleted = $this->client->deleteFile($path);

        return $deleted;
    }
}